fn should_cache_builder(entry: &FileBuilderEntry) -> boolExpand description
Returns true if a freshly pruned builder should be cached for this file.
fn should_cache_builder(entry: &FileBuilderEntry) -> boolReturns true if a freshly pruned builder should be cached for this file.