Pruner

Struct Pruner 

pub struct Pruner {
    worker_senders: Vec<Sender<PruneRequest>>,
    inner: Arc<PrunerInner>,
}
A pruner that prunes files for all partitions of a scanner.

Fields

worker_senders: Vec<Sender<PruneRequest>>

Channels to send requests to workers.

inner: Arc<PrunerInner>

Implementations

Source

impl Pruner

pub fn new(stream_ctx: Arc<StreamContext>, num_workers: usize) -> Self

Creates a new Pruner with N worker tasks.

Initially all file_entries have remaining_ranges = 0. Call add_partition_ranges() to initialize ref counts.

pub fn new_with_options( stream_ctx: Arc<StreamContext>, num_workers: usize, options: PrunerOptions, ) -> Self

Creates a new pruner with the given options.

pub fn add_partition_ranges(&self, partition_ranges: &[PartitionRange])

Adds reference counts for partition ranges that the caller will scan.

The caller must keep these ranges consistent with the ranges passed to PartitionPruner. If called multiple times, all calls must describe ranges from the same logical scan because their reference counts accumulate in this pruner.

pub async fn build_file_ranges( &self, index: RowGroupIndex, pre_filter_mode: PreFilterMode, partition_metrics: &PartitionMetrics, reader_metrics: &mut ReaderMetrics, ) -> Result<SmallVec<[FileRange; 2]>>

Gets or creates the FileRangeBuilder for a file, builds ranges, and decrements its ref count. Non-retained builders are cleaned up at zero.

Callers should invoke add_partition_ranges() to initialize the file entries and ref counts.

pub fn skip_file_range( &self, index: RowGroupIndex, reader_metrics: &mut ReaderMetrics, )

Skips a file range that has been pruned before entering the file pruner.

This keeps the pruner’s per-file reference counts balanced with add_partition_ranges(). It may also clear a cached builder when this was the last remaining range for the file.

async fn get_file_builder( &self, file_index: usize, pre_filter_mode: PreFilterMode, partition_metrics: &PartitionMetrics, reader_metrics: &mut ReaderMetrics, ) -> Result<Arc<FileRangeBuilder>>

Gets or creates the FileRangeBuilder for a file.

pub fn get_file_builder_background( &self, file_index: usize, pre_filter_mode: PreFilterMode, partition_metrics: Option<PartitionMetrics>, )

Gets or creates the FileRangeBuilder for a file.

pub fn predicate_prefilter_enabled(&self) -> bool

Returns whether this pruner builds file ranges with the reduced-column predicate prefilter enabled.

fn get_worker_idx(&self, file_id: FileId) -> usize

async fn prune_file_directly( &self, file_index: usize, pre_filter_mode: PreFilterMode, reader_metrics: &mut ReaderMetrics, ) -> Result<Arc<FileRangeBuilder>>

Prunes a file directly without going through a worker. Used as fallback when worker channels are closed.

fn decrement_and_maybe_clear( &self, file_index: usize, reader_metrics: &mut ReaderMetrics, )

Decrements ref count and clears builder if no longer needed.

async fn worker_loop( worker_id: usize, rx: Receiver<PruneRequest>, inner: Arc<PrunerInner>, )

Worker loop that processes prune requests.